package Hexel;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import javax.media.opengl.awt.GLCanvas;
import javax.swing.JFrame;

import Hexel.chunk.ChunkVisibilityManager;
import Hexel.chunk.Chunks;
import Hexel.math.Vector3i;
import Hexel.rendering.Camera;
import Hexel.rendering.Renderer;
import Hexel.things.Player;
import Hexel.things.Thing;
import Hexel.things.ThingBridge;
import Hexel.things.ThingSimulator;
import Hexel.util.FrameRateTracker;
import Hexel.util.StageChangeListener;
import Hexel.util.TickHandler;

public class Engine {

	private Chunks chunks;
	private Renderer renderer;
	private Camera camera;

	private Stage stage;

	private FrameRateTracker fps;
	private ThingSimulator thingSimulator;
	private WorldSimulator worldSimulator;
	private ChunkVisibilityManager chunkVisibilityManager;

	public JFrame getFrame() {
		return this.renderer.window.getFrame();
	}

	public GLCanvas getCanvas() {
		return this.renderer.window.getCanvas();
	}

	public void start(TickHandler handler) {
		this.renderer.start(handler);
	}

	public ThingBridge getThingBridge() {
		return this.thingSimulator.getThingBridge();
	}

	public void setPlayer(Player player) {
		this.renderer.setPlayer(player);
	}

	private StageChangeListener scl;

	public void setStage(Stage stage) {
		this.stage = stage;
		this.renderer.setStage(stage);
		this.scl.stageChanged(stage);
	}

	public void addThing(Thing thing) {
		this.thingSimulator.addThing(thing);
		this.renderer.addThing(thing);
	}

	public Engine(StageChangeListener scl) {
		this.scl = scl;
		Cleanup cleanup = new Cleanup();
		this.chunks = new Chunks(cleanup, 10 * 10 * 10);
		this.renderer = new Renderer(this.chunks);
		this.fps = new FrameRateTracker(5);
		this.chunkVisibilityManager = new ChunkVisibilityManager(this.chunks, this.renderer);
		this.thingSimulator = new ThingSimulator(this.chunks);
		this.worldSimulator = new WorldSimulator(this.chunks, this.chunkVisibilityManager);
		setStage(Stage.LOADING);

		Runtime.getRuntime().addShutdownHook(cleanup);

	}

	public void setCamera(Camera camera) {
		this.camera = camera;
		this.renderer.setCamera(camera);
	}

	private int i = 0;

	public void step() {
		this.fps.hit();
		if (this.stage == Stage.INGAME){
			this.chunkVisibilityManager.updateVisibility(this.camera);
			if (this.fps.avg() >= 1) {
				this.thingSimulator.step(this.fps.avg());
				this.worldSimulator.step();
			}
		} else {
			if (this.i > 10)
				setStage(Stage.INGAME);
		}
		this.i++;
	}

}
