package Hexel;

import Hexel.blocks.types.BlockDebug;
import Hexel.blocks.types.BlockDirt;
import Hexel.blocks.types.BlockGlass;
import Hexel.blocks.types.BlockLeaf;
import Hexel.blocks.types.BlockStone;
import Hexel.blocks.types.BlockWater;
import Hexel.blocks.types.BlockWood;
import Hexel.math.HexGeometry;
import Hexel.math.Vector2d;
import Hexel.rendering.Camera;
import Hexel.things.Player;
import Hexel.util.FrameRateTracker;
import Hexel.util.StageChangeListener;
import Hexel.util.TickHandler;

public class Hexel implements TickHandler, StageChangeListener {
	public static void main(String[] args) {
		new Hexel();
	}

	public static final String workingDir = System.getProperty("user.dir");

	public static Hexel instance;

	private PCInput input;

	private Player player;

	private Engine engine;

	public static final boolean save = false;

	private int canPlace = 0;

	public void setCamera(Camera camera) {
		this.engine.setCamera(camera);
	}

	private FrameRateTracker fps;

	private Config config;

	public Hexel() {
		System.out.println("Starting up...");
		instance = this;

		this.config = new Config();

		this.engine = new Engine(this);
		int cx = 0;// 61;
		int cy = 0;// 73;
		int cz = 1;

		Vector2d pos = new Vector2d();
		HexGeometry.hexToCartesian(cx * 16, cy * 16, pos);

		double x = pos.x + 4;
		double y = pos.y + 4;
		double z = cz * 16;

		// START 1.9087976974108996 1.2740837857172846 0.0 | 3 1 0

		this.player = new Player(x, y, z, -45, 0, this.engine.getThingBridge());
		this.engine.setPlayer(this.player);
		this.player.blockToPlace = BlockDebug.class;
		this.input = new PCInput(this.engine.getFrame(), this.engine.getCanvas());
		setCamera(this.player);
		this.fps = new FrameRateTracker(5);
		this.engine.start(this);
	}

	@Override
	public void onTick() {
		this.fps.hit();
		updateControls();
		this.engine.step();
	}

	@Override
	public void stageChanged(Stage s) {
		if (s == Stage.INGAME) {
			this.engine.addThing(this.player);
		}
	}

	public void updateControls() {
		if (this.canPlace > 0) {
			this.canPlace--;
		}

		while (!this.input.keysTapped.isEmpty()) {
			int keyTapped = this.input.keysTapped.poll();
			if (keyTapped == this.config.controls.get("toggleFocus")) {
				this.input.toggleCursorObject();
			} else if (keyTapped == this.config.controls.get("gotoDebugBlock")) {
				this.player.blockToPlace = BlockDebug.class;
			} else if (keyTapped == this.config.controls.get("gotoStoneBlock")) {
				this.player.blockToPlace = BlockStone.class;
			} else if (keyTapped == this.config.controls.get("gotoWaterBlock")) {
				this.player.blockToPlace = BlockWater.class;// WaterBlock.blocks[0][8];
			} else if (keyTapped == this.config.controls.get("gotoWoodBlock")) {
				this.player.blockToPlace = BlockWood.class;// WaterBlock.blocks[0][8];
			} else if (keyTapped == this.config.controls.get("gotoLeafBlock")) {
				this.player.blockToPlace = BlockLeaf.class;// WaterBlock.blocks[0][8];
			} else if (keyTapped == this.config.controls.get("gotoGlassBlock")) {
				this.player.blockToPlace = BlockGlass.class;
			} else if (keyTapped == this.config.controls.get("gotoDirtBlock")) {
				this.player.blockToPlace = BlockDirt.class;
			} else if (keyTapped == this.config.controls.get("createBlock")) {
				this.player.createBlock = true;
			}
		}

		double speed = 4;
		if (this.input.keysPressed.contains(this.config.controls.get("sprint"))) {
			speed *= 4.5;
		}

		if (this.input.keysPressed.contains(this.config.controls.get("forward"))) {
			this.player.setForwardSpeed(1 * speed);
		} else if (this.input.keysPressed.contains(this.config.controls.get("backward"))) {
			this.player.setForwardSpeed(-1 * speed);
		} else {
			this.player.setForwardSpeed(0);
		}

		if (this.input.keysPressed.contains(this.config.controls.get("left"))) {
			this.player.setSidewaysSpeed(-1 * speed);
		} else if (this.input.keysPressed.contains(this.config.controls.get("right"))) {
			this.player.setSidewaysSpeed(1 * speed);
		} else {
			this.player.setSidewaysSpeed(0);
		}

		this.player.deleteBlock = this.input.keysPressed.contains(this.config.controls
				.get("removeBlock"));

		this.player.createBlock = this.player.createBlock
				|| this.input.placeBlock && this.canPlace == 0;
		this.player.deleteBlock = this.player.deleteBlock || this.input.breakBlock;
		if (this.player.createBlock) {
			this.canPlace = 10;
		}

		if (this.input.keysPressed.contains(this.config.controls.get("jump"))) {
			this.player.jump(4);
		}

		double dx = this.input.getDX() / 5.0;
		double dy = this.input.getDY() / 5.0;

		this.player.lookVertical(dy);
		this.player.lookHorizontal(dx);
	}

}
