package Hexel;

import java.awt.AWTException;
import java.awt.Cursor;
import java.awt.Image;
import java.awt.Point;
import java.awt.Robot;
import java.awt.Toolkit;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.image.MemoryImageSource;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Queue;

import javax.media.opengl.awt.GLCanvas;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;

public class PCInput {

	public double frontDir;
	public double sideDir;

	public HashSet<Integer> keysPressed = new HashSet<Integer>();
	public Queue<Integer> keysTapped = new LinkedList<Integer>();

	private JFrame frame;
	private GLCanvas canvas;

	public double dx;
	public double dy;

	private int[] cx = new int[2];
	private int[] cy = new int[2];

	private Robot robot;

	private boolean cursorObject = false;
	private boolean cursorHidden = false;

	public boolean placeBlock = false;
	public boolean breakBlock = false;

	public PCInput(JFrame frame, GLCanvas canvas) {
		this.frame = frame;
		this.canvas = canvas;

		try {
			this.robot = new Robot();
		} catch (AWTException e) {
			e.printStackTrace();
		}

		this.dx = 0;
		this.dy = 0;

		this.frontDir = 0;
		this.sideDir = 0;

		canvas.addKeyListener(new KeyAdapter() {
			@Override
			public void keyPressed(KeyEvent e) {
				handleKeyPressed(e);
			}

			@Override
			public void keyReleased(KeyEvent e) {
				handleKeyReleased(e);
			}
		});
		canvas.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseExited(MouseEvent e) {
				handleMouseExited(e);
			}

			@Override
			public void mousePressed(MouseEvent e) {
				handleMousePressed(e);
			}

			@Override
			public void mouseReleased(MouseEvent e) {
				handleMouseReleased(e);
			}
		});
		canvas.addMouseMotionListener(new MouseMotionAdapter() {
			@Override
			public void mouseMoved(MouseEvent e) {
				handleMouseMoved(e);
			}
		});
	}

	public double getDX() {
		double tmp = this.dx;
		this.dx = 0;
		return tmp;
	}

	public double getDY() {
		double tmp = this.dy;
		this.dy = 0;
		return tmp;
	}

	public void handleKeyPressed(KeyEvent e) {
		this.keysPressed.add(e.getKeyCode());
		this.keysTapped.add(e.getKeyCode());
	}

	public void handleKeyReleased(KeyEvent e) {
		this.keysPressed.remove(e.getKeyCode());
	}

	public void handleMouseExited(MouseEvent e) {
		if (this.frame.isActive() && this.cursorObject) {
			this.robot.mouseMove(
					this.frame.getX() + this.canvas.getWidth() / 2
					+ this.frame.getInsets().left,
					this.frame.getY() + this.canvas.getHeight() / 2
					+ this.frame.getInsets().top);
			this.cx[0] = this.canvas.getWidth() / 2;
			this.cy[0] = this.canvas.getHeight() / 2;
			this.cx[1] = this.cx[0];
			this.cy[1] = this.cy[0];
		}
	}

	public void handleMouseMoved(MouseEvent e) {
		if (this.cursorObject) {
			if (e.getX() < 30 || e.getX() > this.canvas.getWidth() - 30
					|| e.getY() < 30 || e.getY() > this.canvas.getHeight() - 30) {
				this.robot.mouseMove(
						this.frame.getX() + this.canvas.getWidth() / 2
						+ this.frame.getInsets().left,
						this.frame.getY() + this.canvas.getHeight() / 2
						+ this.frame.getInsets().top);
				this.cx[0] = this.canvas.getWidth() / 2;
				this.cy[0] = this.canvas.getHeight() / 2;
				this.cx[1] = this.cx[0];
				this.cy[1] = this.cy[0];
			} else {
				this.cx[1] = this.cx[0];
				this.cy[1] = this.cy[0];
				this.cx[0] = e.getX();
				this.cy[0] = e.getY();

				this.dx += this.cx[1] - this.cx[0];
				this.dy += this.cy[1] - this.cy[0];
			}
		}
	}

	public void handleMousePressed(MouseEvent e) {
		this.breakBlock = SwingUtilities.isLeftMouseButton(e);
		this.placeBlock = SwingUtilities.isRightMouseButton(e);
	}

	public void handleMouseReleased(MouseEvent e) {
		this.breakBlock = !SwingUtilities.isLeftMouseButton(e) && this.breakBlock;
		this.placeBlock = !SwingUtilities.isRightMouseButton(e) && this.placeBlock;
	}

	private void hideCursor() {
		int[] pixels = new int[16 * 16];
		Image image = Toolkit.getDefaultToolkit().createImage(
				new MemoryImageSource(16, 16, pixels, 0, 16));
		Cursor transparentCursor = Toolkit.getDefaultToolkit()
				.createCustomCursor(image, new Point(0, 0), "invisibleCursor");
		this.frame.setCursor(transparentCursor);
		this.cursorHidden = true;
	}

	private void showCursor() {
		this.frame.setCursor(null);
		this.cursorHidden = false;
	}

	public void toggleCursorObject() {
		this.cursorObject = !this.cursorObject;
		if (this.cursorHidden)
			showCursor();
		else {
			hideCursor();
		}
	}
}
