package Hexel;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import Hexel.blocks.BlockDelta;
import Hexel.blocks.BlockRules;
import Hexel.blocks.types.Block;
import Hexel.chunk.Chunk;
import Hexel.chunk.Chunks;
import Hexel.chunk.LoadedChunksGetter;
import Hexel.math.Vector2i;
import Hexel.math.Vector3i;
import Hexel.util.Pair;

public class WorldSimulator {

	private int steps = 0;

	private Chunks chunks;

	private LoadedChunksGetter loadedChunksGetter;

	private BlockRules blockRules;

	public WorldSimulator(Chunks chunks, LoadedChunksGetter loadedChunksGetter) {
		this.chunks = chunks;
		this.loadedChunksGetter = loadedChunksGetter;
		this.blockRules = new BlockRules(chunks);
	}

	private ExecutorService ste = Executors.newSingleThreadExecutor();
	public void step() {
		this.ste.execute(new Runnable() {
			@Override
			public void run() {
				sim();
			}
		});

	}
	
	public void sim(){
		final ArrayList<Vector3i> toUpdate = new ArrayList<Vector3i>();
		HashSet<Vector3i> loadedChunks = this.loadedChunksGetter.getLoadedChunks();
		for (Vector3i pos : loadedChunks) {
			toUpdate.add(pos);
		}
		
		boolean goAgain = false;

		do {
			ArrayList<BlockDelta> deltas = new ArrayList<BlockDelta>();

			goAgain = false;
			for (Vector3i pos : toUpdate) {
				Chunk chunk = WorldSimulator.this.chunks.getChunk(pos);
				chunk.beingUpdated = true;
				stepChunk(pos, deltas);
				if (chunk.stepsToNeedSim == -1)
					goAgain = true;
			}

			Vector3i tmp = new Vector3i();
			Vector2i tmp2 = new Vector2i();
			for (BlockDelta delta : deltas) {
				if (!delta.immediate){
					WorldSimulator.this.chunks.setBlock(delta.x, delta.y, delta.z, delta.block, tmp, tmp2);
					if (delta.stepsToSim > 0)
						WorldSimulator.this.chunks.setStepsToSim(delta.x, delta.y, delta.z, delta.stepsToSim-1, tmp);
					else
						WorldSimulator.this.chunks.setStepsToSim(delta.x, delta.y, delta.z, delta.stepsToSim, tmp);
					if (delta.stepsToSim-1 > 0){
						goAgain = true;
					}
				}
			}
			for (Vector3i pos : toUpdate) {
				Chunk chunk = WorldSimulator.this.chunks.getChunk(pos);
				chunk.beingUpdated = false;
			}
		} while (goAgain);
		WorldSimulator.this.steps += 1;
	}

	public void stepChunk(Vector3i p, ArrayList<BlockDelta> deltas) {
		Chunk chunk = this.chunks.getChunk(p);

		if (chunk.stepsToNeedSim != 0 && chunk.stepsToNeedSim != -1) {
			if (chunk.stepsToNeedSim != Integer.MAX_VALUE)
				chunk.stepsToNeedSim -= 1;
			return;
		}
		chunk.stepsToNeedSim = Integer.MAX_VALUE;

		Vector3i tmp = new Vector3i();
		Vector2i tmp2 = new Vector2i();

		int cx = p.x;
		int cy = p.y;
		int cz = p.z;

		// System.out.println(("+======+"));

		for (int x = 0; x < 32; x++) {
			for (int y = 0; y < 16; y++) {
				for (int z = 0; z < 32; z++) {
					Block b = chunk.get(x, y, z);
					int gx = cx * 32 + x;
					int gy = cy * 16 + y;
					int gz = cz * 32 + z;

					Pair<Integer, ArrayList<BlockDelta>> pair = this.blockRules
							.apply(gx, gy, gz, chunk, b, this.steps, tmp, tmp2);
					ArrayList<BlockDelta> additionalDeltas = pair.second;
					chunk.stepsToNeedSim = Math.min(pair.first, chunk.stepsToNeedSim);
					if (additionalDeltas != null)
						deltas.addAll(additionalDeltas);
				}
			}
		}
//		if (p.x == 0 && p.y == 0 && p.z == 0)
//			System.out.println((chunk.getXYZ())+ " " + ("|")+ " " + (steps)+ " " + ("|")+ " " + (chunk.stepsToNeedSim));
	}
}
