package Hexel.blocks.rules;

import java.util.ArrayList;

import Hexel.blocks.BlockDelta;
import Hexel.blocks.BlockRule;
import Hexel.blocks.FreqBlockRule;
import Hexel.blocks.types.Block;
import Hexel.blocks.types.BlockDirt;
import Hexel.blocks.types.BlockWood;
import Hexel.blocks.types.BlockWoodData;
import Hexel.chunk.Chunk;
import Hexel.chunk.Chunks;
import Hexel.math.HexGeometry;
import Hexel.math.Vector2i;
import Hexel.math.Vector3i;

public class GrassGrowRule  extends FreqBlockRule {
	
	public GrassGrowRule() {
		super(250);
	}

	@Override
	public boolean isMatch(int x, int y, int z, Block b, Chunk c,
			Chunks chunks, int steps, Vector3i tmp, Vector2i tmp2) {
		BlockDirt bd = (BlockDirt)b;
		if (!bd.hasGrass)
			return false;
		Vector2i[] neighbors = HexGeometry.getNeighbors(x);
		for (int i = 0; i < neighbors.length; i++){
			int nx = x + neighbors[i].x;
			int ny = y + neighbors[i].y;
			Block nb = chunks.getBlock(nx, ny, z, tmp);
			if (nb instanceof BlockDirt && !((BlockDirt)nb).hasGrass)
				return true;
		}
		return false;
	}

	@Override
	public ArrayList<BlockDelta> apply(int x, int y, int z, Block b,
			Chunk c, Chunks chunks, int steps, Vector3i tmp, Vector2i tmp2) {
		ArrayList<BlockDelta> deltas = new ArrayList<BlockDelta>();
		

		Vector2i[] neighbors = HexGeometry.getNeighbors(x);
		for (int i = 0; i < neighbors.length; i++){
			int nx = x + neighbors[i].x;
			int ny = y + neighbors[i].y;
			Block nb = chunks.getBlock(nx, ny, z, tmp);
			if (nb instanceof BlockDirt && !((BlockDirt)nb).hasGrass){
				HexGeometry.globalToLocal(x, y, z, tmp);
				deltas.add(new BlockDelta(nx, ny, z, BlockDirt.Make(nb.getHealth(), true), false, c.stepsToSim[tmp.x][tmp.y][tmp.z]));
			}
		}
		return deltas;
	}

}
