package Hexel.blocks.rules;

import java.util.ArrayList;

import Hexel.blocks.BlockDelta;
import Hexel.blocks.BlockRule;
import Hexel.blocks.BlockTools;
import Hexel.blocks.types.Block;
import Hexel.blocks.types.BlockTransparent;
import Hexel.chunk.Chunk;
import Hexel.chunk.Chunks;
import Hexel.math.HexGeometry;
import Hexel.math.Vector2i;
import Hexel.math.Vector3i;

public class LightRule implements BlockRule {

	@Override
	public int stepsUntilConsider(int x, int y, int z, Block b,
			Chunk c, Chunks chunks, int steps, Vector3i tmp, Vector2i tmp2) {
		return 0;
	}

	@Override
	public boolean isMatch(int x, int y, int z, Block b, Chunk c,
			Chunks chunks, int steps, Vector3i tmp, Vector2i tmp2) {
		BlockTransparent be = (BlockTransparent)b;
		int newLightLevel = 0;
		if (z > chunks.getHighestBlockAtXY(x, y, tmp2)){
			newLightLevel = BlockTransparent.MAX_LIGHT_LEVEL;
		}
		else {
			Vector3i[] neighbors;
			if (x % 2 == 0) {
				neighbors = HexGeometry.evenAllNeighbors;
			} else {
				neighbors = HexGeometry.oddAllNeighbors;
			}
			int highestNeighborLightLevel = 0;
			for (Vector3i neighbor2 : neighbors) {
				Block neighbor = chunks.getBlock(x+neighbor2.x, y+neighbor2.y, z+neighbor2.z, tmp);
				if (neighbor instanceof BlockTransparent){
					BlockTransparent neighborE = (BlockTransparent)neighbor;
					highestNeighborLightLevel = Math.max(highestNeighborLightLevel, neighborE.getLightLevel()-1-neighborE.occlusion);
				}
			}
			Block neighbor = chunks.getBlock(x, y, z+1, tmp);
			if (neighbor instanceof BlockTransparent){
				BlockTransparent neighborE = (BlockTransparent)neighbor;
				highestNeighborLightLevel = Math.max(highestNeighborLightLevel, neighborE.getLightLevel()-neighborE.occlusion);
			}
			neighbor = chunks.getBlock(x, y, z-1, tmp);
			if (neighbor instanceof BlockTransparent){
				BlockTransparent neighborE = (BlockTransparent)neighbor;
				highestNeighborLightLevel = Math.max(highestNeighborLightLevel, neighborE.getLightLevel()-1-neighborE.occlusion);
			}
			newLightLevel = highestNeighborLightLevel;
		}
		if (newLightLevel !=  be.lightLevel){
			be.nextLightLevel = newLightLevel;
			return true;
		}
		return false;
	}

	@Override
	public ArrayList<BlockDelta> apply(int x, int y, int z, Block b,
			Chunk c, Chunks chunks, int steps, Vector3i tmp, Vector2i tmp2) {

		BlockTransparent be = (BlockTransparent)chunks.getBlock(x, y, z, tmp);
		BlockTransparent newBE = BlockTools.updateLightLevel(be, be.nextLightLevel);
		ArrayList<BlockDelta> deltas = new ArrayList<BlockDelta>();
		HexGeometry.globalToLocal(x, y, z, tmp);
		deltas.add(new BlockDelta(x, y, z, newBE, true, c.stepsToSim[tmp.x][tmp.y][tmp.z]));
		return deltas;
	}

}
