package Hexel.blocks.rules;

import java.util.ArrayList;
import java.util.Random;

import Hexel.blocks.BlockDelta;
import Hexel.blocks.BlockRule;
import Hexel.blocks.FreqBlockRule;
import Hexel.blocks.types.Block;
import Hexel.blocks.types.BlockEmpty;
import Hexel.blocks.types.BlockLeaf;
import Hexel.blocks.types.BlockWood;
import Hexel.blocks.types.BlockWoodData;
import Hexel.chunk.Chunk;
import Hexel.chunk.Chunks;
import Hexel.math.HexGeometry;
import Hexel.math.Vector2i;
import Hexel.math.Vector3i;
import Hexel.util.Util;

public class TreeRules {
	
	public static void addRules(ArrayList<BlockRule> woodRules, ArrayList<BlockRule> leafRules){
		for (int _neighborIndex = 0; _neighborIndex < 5; _neighborIndex++){
			final int neighborIndex = _neighborIndex;
			
			//==========BRANCHES=========
			woodRules.add(new FreqBlockRule(1000){
				@Override
				public boolean isMatch(int x, int y, int z, Block b,
						Chunk c, Chunks chunks, int steps, Vector3i tmp, Vector2i tmp2) {
					Vector3i[] neighbors = HexGeometry.getAllNeighbors(x);
					Vector3i neighbor = neighbors[neighborIndex];
					BlockWood wb = (BlockWood) b;
					if (!wb.data.alive)
						return false;
					Block nb = chunks.getBlock(x + neighbor.x, y + neighbor.y, z + neighbor.z, tmp);
					if (!(nb instanceof BlockEmpty || nb instanceof BlockLeaf))
						return false;
					
					int nx = x + neighbor.x;
					int ny = y + neighbor.y;
					int nz = z + neighbor.z;
					
					boolean isBranch = nx != x || ny != y;

					if (nz < z)
						return false;
					if(wb.data.level < 4 && isBranch)
						return false;
	
					int h = 1;
					h = 31*h + nx;
					h = 31*h + ny;
					h = 31*h + nz;
					
					if (wb.data.level != 0){
						double chanceGrow = 0;
						if (isBranch){
							chanceGrow = 	(	.5*((wb.data.level-wb.data.branchLevel)%2) +
												.05*(wb.data.level-10)
										 	)
										 	*(1-wb.data.branchLevel*.25)
										 	*(1-wb.data.branches*.25);
						}
						else {
							chanceGrow = 3.0 - .1*wb.data.level*(wb.data.branchLevel*10000+1);
						}
						if (Util.hashToDouble(h) > chanceGrow)
							return false;
					}
					
					return true;
				}

				@Override
				public ArrayList<BlockDelta> apply(int x, int y, int z,
						Block b, Chunk c, Chunks chunks, int steps,
						Vector3i tmp, Vector2i tmp2) {
					ArrayList<BlockDelta> deltas = new ArrayList<BlockDelta>();
					BlockWood wb = (BlockWood) b;
					
					Vector3i[] neighbors = HexGeometry.getAllNeighbors(x);
					Vector3i neighbor = neighbors[neighborIndex];
					
					int nx = x + neighbor.x;
					int ny = y + neighbor.y;
					int nz = z + neighbor.z;
					
					boolean isBranch = nx != x || ny != y;

					BlockWood grownWB;
					BlockWood newWB;
					if (isBranch){
						grownWB = BlockWood.Make(new BlockWoodData(BlockWood.MAX_HEALTH, wb.data.level+1, wb.data.branchLevel+1, 0, true));
						newWB = BlockWood.Make(new BlockWoodData(BlockWood.MAX_HEALTH, wb.data.level, wb.data.branchLevel, wb.data.branches+1, true));
					}
					else {
						grownWB = BlockWood.Make(new BlockWoodData(BlockWood.MAX_HEALTH, wb.data.level+1, wb.data.branchLevel, 0, true));
						newWB = BlockWood.Make(new BlockWoodData(BlockWood.MAX_HEALTH, wb.data.level, wb.data.branchLevel, wb.data.branches, true));
					}
					HexGeometry.globalToLocal(x, y, z, tmp);
					deltas.add(new BlockDelta(nx, ny, nz, grownWB, false, c.stepsToSim[tmp.x][tmp.y][tmp.z]));
					deltas.add(new BlockDelta(x, y, z, newWB, true, c.stepsToSim[tmp.x][tmp.y][tmp.z]));
					return deltas;
				}
			});

			//==========LEAFS=========
			woodRules.add(new FreqBlockRule(500){
				@Override
				public boolean isMatch(int x, int y, int z, Block b,
						Chunk c, Chunks chunks, int steps, Vector3i tmp, Vector2i tmp2) {
					Vector3i[] neighbors = HexGeometry.getAllNeighbors(x);
					Vector3i neighbor = neighbors[neighborIndex];
					BlockWood wb = (BlockWood) b;
					if (!wb.data.alive)
						return false;
					if(wb.data.level < 4)
						return false;
					if (!(chunks.getBlock(x + neighbor.x, y + neighbor.y, z + neighbor.z, tmp) instanceof BlockEmpty))
						return false;
					
					int nx = x + neighbor.x;
					int ny = y + neighbor.y;
					int nz = z + neighbor.z;
	
					int h = 1;
					h = 31*h + nx;
					h = 31*h + ny;
					h = 31*h + nz;
					if (wb.data.level != 0){
						double chanceGrow = 0;
						chanceGrow = Math.min(Math.max(.1*(wb.data.level-5), 0), .75) + Math.min(.5*wb.data.branchLevel, .75);
						if (Util.hashToDouble(h) > chanceGrow)
							return false;
					}
					return true;
				}
	
				@Override
				public ArrayList<BlockDelta> apply(int x, int y, int z,
						Block b, Chunk c, Chunks chunks, int steps,
						Vector3i tmp, Vector2i tmp2) {
					ArrayList<BlockDelta> deltas = new ArrayList<BlockDelta>();
					BlockWood wb = (BlockWood) b;
					
					Vector3i[] neighbors = HexGeometry.getAllNeighbors(x);
					Vector3i neighbor = neighbors[neighborIndex];
					
					int nx = x + neighbor.x;
					int ny = y + neighbor.y;
					int nz = z + neighbor.z;

					BlockLeaf leaf = BlockLeaf.Make(0, BlockLeaf.MAX_HEALTH);
					HexGeometry.globalToLocal(x, y, z, tmp);
					deltas.add(new BlockDelta(nx, ny, nz, leaf, false, c.stepsToSim[tmp.x][tmp.y][tmp.z]));
					return deltas;
				}
			});
			leafRules.add(new FreqBlockRule(250){
				@Override
				public boolean isMatch(int x, int y, int z, Block b,
						Chunk c, Chunks chunks, int steps, Vector3i tmp, Vector2i tmp2) {
					Vector3i[] neighbors = HexGeometry.getAllNeighbors(x);
					Vector3i neighbor = neighbors[neighborIndex];
					BlockLeaf lb = (BlockLeaf) b;
					if (!(chunks.getBlock(x + neighbor.x, y + neighbor.y, z + neighbor.z, tmp) instanceof BlockEmpty))
						return false;
					
					int nx = x + neighbor.x;
					int ny = y + neighbor.y;
					int nz = z + neighbor.z;
	
					int h = 1;
					h = 31*h + nx;
					h = 31*h + ny;
					h = 31*h + nz;
					double chanceGrow = .1;
					if (Util.hashToDouble(h) > chanceGrow)
						return false;
					return true;
				}
	
				@Override
				public ArrayList<BlockDelta> apply(int x, int y, int z,
						Block b, Chunk c, Chunks chunks, int steps,
						Vector3i tmp, Vector2i tmp2) {
					ArrayList<BlockDelta> deltas = new ArrayList<BlockDelta>();
					
					Vector3i[] neighbors = HexGeometry.getAllNeighbors(x);
					Vector3i neighbor = neighbors[neighborIndex];
					
					int nx = x + neighbor.x;
					int ny = y + neighbor.y;
					int nz = z + neighbor.z;

					BlockLeaf leaf = BlockLeaf.Make(0, BlockLeaf.MAX_HEALTH);
					HexGeometry.globalToLocal(x, y, z, tmp);
					deltas.add(new BlockDelta(nx, ny, nz, leaf, false, c.stepsToSim[tmp.x][tmp.y][tmp.z]));
					return deltas;
				}
			});
		}
		
	}

}
