package Hexel.blocks.rules;

import java.util.ArrayList;

import Hexel.blocks.BlockDelta;
import Hexel.blocks.BlockRule;
import Hexel.blocks.types.Block;
import Hexel.blocks.types.BlockEmpty;
import Hexel.blocks.types.BlockWater;
import Hexel.chunk.Chunk;
import Hexel.chunk.Chunks;
import Hexel.math.HexGeometry;
import Hexel.math.Vector2i;
import Hexel.math.Vector3i;

public class WaterfallRule implements BlockRule {
	@Override
	public int stepsUntilConsider(int x, int y, int z, Block b, Chunk c,
			Chunks chunks, int steps, Vector3i tmp, Vector2i tmp2) {
		return (3 - steps % 3) % 3;
	}

	@Override
	public boolean isMatch(int x, int y, int z, Block b, Chunk c,
			Chunks chunks, int steps, Vector3i tmp, Vector2i tmp2) {
		BlockWater wb = (BlockWater) b;
		Block below = chunks.getBlock(x, y, z - 1, tmp);
		if (wb.getBottom() != 0)
			return true;
		if (below instanceof BlockEmpty)
			return true;
		if (below instanceof BlockWater) {
			BlockWater wBelow = (BlockWater) below;
			if (wBelow.getTop() < 8)
				return true;
		}
		return false;
	}

	@Override
	public ArrayList<BlockDelta> apply(int x, int y, int z, Block b, Chunk c,
			Chunks chunks, int steps, Vector3i tmp, Vector2i tmp2) {

		ArrayList<BlockDelta> deltas = new ArrayList<BlockDelta>();
		BlockWater wb = (BlockWater) b;

		int wbBottom = wb.getBottom() - 4;
		int wbTop = wb.getTop() - 4;

		if (wbBottom >= 0) {
			HexGeometry.globalToLocal(x, y, z, tmp);
			deltas.add(new BlockDelta(x, y, z, BlockWater.Make(wb.lightLevel, wbBottom, wbTop), true, c.stepsToSim[tmp.x][tmp.y][tmp.z]));
		} else {
			Block below = chunks.getBlock(x, y, z - 1, tmp);
			if (below instanceof BlockEmpty) {
				int top = Math.min(8, 8 + wbTop);
				int bottom = 8 + wbBottom;
				BlockWater replacement = BlockWater.Make(wb.lightLevel, bottom, top);
				HexGeometry.globalToLocal(x, y, z, tmp);
				deltas.add(new BlockDelta(x, y, z - 1, replacement, true, c.stepsToSim[tmp.x][tmp.y][tmp.z]));
			} else if (below instanceof BlockWater) {
				BlockWater belowWB = (BlockWater) below;
				int h = Math.min(wbTop, 0) - wbBottom;
				int top = Math.min(8, belowWB.getTop() + h);
				BlockWater replacement = BlockWater.Make(wb.lightLevel, belowWB.getBottom(),
						top);
				HexGeometry.globalToLocal(x, y, z, tmp);
				deltas.add(new BlockDelta(x, y, z - 1, replacement, true, c.stepsToSim[tmp.x][tmp.y][tmp.z]));
			}

			if (wbTop <= 0) {
				HexGeometry.globalToLocal(x, y, z, tmp);
				deltas.add(new BlockDelta(x, y, z, BlockEmpty.Make(wb.lightLevel), true, c.stepsToSim[tmp.x][tmp.y][tmp.z]));
			} else {
				wbBottom = 0;
				HexGeometry.globalToLocal(x, y, z, tmp);
				deltas.add(new BlockDelta(x, y, z, BlockWater.Make(wb.lightLevel, wbBottom, wbTop), true, c.stepsToSim[tmp.x][tmp.y][tmp.z]));
			}
		}
		return deltas;

	}
}
