package Hexel.generation.terrainGenerator.originalPlateGenerator.heightMap;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import Hexel.Cleanup;
import Hexel.math.Vector2i;

public class SmoothHeightMapChunks {
	private ConcurrentHashMap<Vector2i, SmoothHeightMapChunk> inMemorySHMCs = new ConcurrentHashMap<Vector2i, SmoothHeightMapChunk>();
	private Object[] locks = new Object[100];

	SmoothHeightMapChunkGenerator shmcg;

	public SmoothHeightMapChunks(Cleanup cleanup) {
		this.shmcg = new SmoothHeightMapChunkGenerator();
		for (int i = 0; i < this.locks.length; i++) {
			this.locks[i] = new Object();
		}
		cleanup.add(new Runnable() {
			@Override
			public void run() {
				saveAll();
			}
		});
	}

	public SmoothHeightMapChunk get(Vector2i ppos) {
		if (hasSHMCInMemory(ppos)) {
			return this.inMemorySHMCs.get(ppos);
		}
		maybeGenSHMC(ppos);
		return this.inMemorySHMCs.get(ppos);
	}

	private void maybeGenSHMC(Vector2i ppos) {
		int hash = Math.abs(ppos.hashCode() % this.locks.length);
		synchronized (this.locks[hash]) {
			if (hasSHMCInMemory(ppos))
				return;

			SmoothHeightMapChunk shmc;
			if (SHMCFile.has(ppos)) {
				shmc = SHMCFile.load(ppos);
			} else {
				shmc = this.shmcg.gen(ppos.x, ppos.y);
			}
			this.inMemorySHMCs.put(new Vector2i(ppos), shmc);
		}
	}

	public void saveAll() {

		for (Map.Entry<Vector2i, SmoothHeightMapChunk> entry : this.inMemorySHMCs
				.entrySet()) {
			if (!SHMCFile.has(entry.getKey())) {
				SHMCFile.save(entry.getKey(), entry.getValue());
			}
		}
	}

	private boolean hasSHMCInMemory(Vector2i ppos) {
		return this.inMemorySHMCs.containsKey(ppos);
	}

	public void unloadSHMC(Vector2i ppos) {
		this.inMemorySHMCs.remove(ppos);
	}

	public int getHeight(int x, int y, Vector2i p) {
		int px = (int) Math.floor(x * 1.0 / SmoothHeightMapChunk.WIDTH);
		int py = (int) Math.floor(y * 1.0 / SmoothHeightMapChunk.HEIGHT);

		x -= px * SmoothHeightMapChunk.WIDTH;
		y -= py * SmoothHeightMapChunk.HEIGHT;

		p.x = px;
		p.y = py;
		return get(p).heights[x][y];
	}

}
