package Hexel.rendering;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Stroke;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.io.InputStream;

import javax.imageio.ImageIO;
import javax.media.opengl.GLAutoDrawable;

import Hexel.Stage;
import Hexel.blocks.types.Block;
import Hexel.things.Player;

import com.jogamp.opengl.util.awt.Overlay;

public class HUD {

	private Overlay overlay;
	private Color TRANSPARENT_COLOR = new Color(0.0f, 0.0f, 0.0f, 0.0f);

	private Color UNDERWATER_COLOR = new Color(0.0f, 0.0f, 1.0f, .5f);

	private Color CROSSHAIR_COLOR = new Color(1.0f, 1.0f, 1.0f, .1f);
	private Stroke CROSSHAIR_STROKE = new BasicStroke(3);

	private Color BLOCK_AVAILABILITY_COLOR = new Color(1.0f, 1.0f, 1.0f, .5f);

	private BufferedImage tex;

	private Player player;

	public void setPlayer(Player player) {
		this.player = player;
	}

	private Stage stage;

	public void setStage(Stage stage) {
		this.stage = stage;
	}

	public HUD() {
		this.tex = TextureAtlas.createAtlas();
	}

	public void init(GLAutoDrawable drawable) {
		this.overlay = new Overlay(drawable);
	}

	private int w;
	private int h;

	public void updateSize(int w, int h) {
		this.w = w;
		this.h = h;
	}

	public void display() {
		Graphics2D page = this.overlay.createGraphics();
		page.setBackground(this.TRANSPARENT_COLOR);
		page.clearRect(0, 0, this.w, this.h);
		if (this.stage == Stage.LOADING) {
			drawLoading(page);
		} else {
			drawUnderwaterOverlay(page);
			drawCrosshairs(page);
			drawCurrentBlock(page);
		}
		this.overlay.markDirty(0, 0, this.w, this.h);
		this.overlay.drawAll();
		page.dispose();
	}

	private void drawLoading(Graphics2D page) {
		page.setColor(Color.BLACK);
		page.fillRect(0, 0, this.w, this.h);
		page.setColor(Color.WHITE);
		page.drawString("loading...", 25, 25);
	}

	private void drawUnderwaterOverlay(Graphics2D page) {
		if (!this.player.isUnderwater())
			return;
		page.setColor(this.UNDERWATER_COLOR);
		page.fillRect(0, 0, this.w, this.h);
	}

	private void drawCrosshairs(Graphics2D page) {
		int cs = Math.min(this.w / 24, this.h / 24);
		int ch = cs * 3 / 4;

		page.setColor(this.CROSSHAIR_COLOR);
		page.setStroke(this.CROSSHAIR_STROKE);
		page.drawLine(this.w / 2 - cs / 2, this.h / 2, this.w / 2 + cs / 2, this.h / 2);
		page.drawLine(this.w / 2, this.h / 2 - ch / 2, this.w / 2, this.h / 2 + ch / 2);
	}

	private void drawCurrentBlock(Graphics2D page) {
		Block b = this.player.getBlockToPlace();
		if (b == null)
			return;
		int availability = this.player.getBlockAvailability(b.getClass());

		int i = b.getTopTextureIndex();
		int texW = this.tex.getWidth() / TextureAtlas.HOR;
		int texH = this.tex.getHeight() / TextureAtlas.VER;
		int texX = i % TextureAtlas.HOR * texW;
		int texY = i / TextureAtlas.HOR * texH;

		BufferedImage subtex = this.tex.getSubimage(texX, texY, texW, texH);

		AffineTransform at = new AffineTransform();
		int iw = Math.max(200, this.h / 5);
		int ih = Math.max(200, this.h / 5);
		at.translate(0, this.h - ih / 2);
		at.scale(iw * 1.0 / subtex.getWidth(), ih * 1.0 / subtex.getHeight());
		at.rotate(Math.PI / 4);
		page.drawImage(subtex, at, null);

		Font font = new Font("Arial", Font.PLAIN, 64);
		page.setFont(font);
		page.setColor(this.BLOCK_AVAILABILITY_COLOR);
		page.drawString("" + availability, texH / 2, this.h - texH / 2);
	}

}
