package Hexel.rendering;

import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import javax.imageio.ImageIO;

import Hexel.util.Util;

public class TextureAtlas {

	public static int HOR;
	public static int VER = 4;
	
	private static final int HOR_TEX = 4;

	/**
	 * Determines the index of a texture at the given location. Oriented from
	 * the top-left of the texture grid. 0-indexed.
	 * 
	 * @param x
	 *            The  column of the texture.
	 * @param y
	 *            The row of the texture.
	 * @return Return the texture index value for the given location.
	 */
	public static int getTextureIndex(int x, int y) {
		return x + y * HOR;
	}

	private static BufferedImage atlas = null;
	
	public static BufferedImage createAtlas(){
		if (TextureAtlas.atlas != null)
			return TextureAtlas.atlas;

		BufferedImage tex = TextureAtlas.LoadBI("/img/tex.png");
		BufferedImage dmg = TextureAtlas.LoadBI("/img/damage.png");
		
		int texCopies = 1+dmg.getWidth()/dmg.getHeight();
		TextureAtlas.HOR = TextureAtlas.HOR_TEX*texCopies;
		
		int dbh = dmg.getHeight();
		int dbw = dbh;
		
		int bw = tex.getWidth()/HOR_TEX;
		int bh = tex.getHeight()/VER;
		
		BufferedImage atlas = new BufferedImage(tex.getWidth()*texCopies, tex.getHeight(), BufferedImage.TYPE_INT_ARGB);
		Graphics2D page = atlas.createGraphics();
		for (int i = 0; i < texCopies; i++){
			page.drawImage(tex, tex.getWidth()*i, 0, null);
			if (i > 0){
				for (int x = 0; x < HOR_TEX; x++){
					for (int y = 0; y < VER; y++){
						page.drawImage(dmg, tex.getWidth()*i + x*bw, y*bh, tex.getWidth()*i + bw*(x+1), bh*(y+1), (i-1)*dbw, 0, dbw*(i), dbh, null, null);
					}
				}
			}
		}
		TextureAtlas.atlas = atlas;
		
        return atlas;
	}
	
	private static BufferedImage LoadBI(String path){
		try {
			if (!Util.inJar())
				return ImageIO.read(new File("src/main/resources" + path));
			else 
				return ImageIO.read(ChunkRenderer.class.getResourceAsStream(path));
		} catch (Exception e) {
			System.out.println("Caught Exception loading image " + path + "!");
			e.printStackTrace();
			System.exit(1);
		}
		return null;
	}

}
