package Hexel.things;

import javax.media.opengl.GL2;

import Hexel.blocks.types.Block;
import Hexel.blocks.types.BlockDebug;
import Hexel.blocks.types.BlockDirt;
import Hexel.blocks.types.BlockGlass;
import Hexel.blocks.types.BlockLeaf;
import Hexel.blocks.types.BlockSeed;
import Hexel.blocks.types.BlockStone;
import Hexel.blocks.types.BlockWater;
import Hexel.blocks.types.BlockWood;
import Hexel.blocks.types.BlockWoodData;
import Hexel.math.Vector3d;
import Hexel.math.Vector3i;
import Hexel.rendering.Camera;
import Hexel.rendering.Face;
import Hexel.rendering.HighlightBlockFace;
import Hexel.rendering.Renderable;
import Hexel.things.blockManipulation.AddBlock;
import Hexel.things.blockManipulation.BlockAction;
import Hexel.things.blockManipulation.BlockManipulator;
import Hexel.things.blockManipulation.RemoveBlock;

public class Player implements Movable, Cuboid, Volumetric, Camera, Thing,
BlockManipulator, Renderable, InventoryOwner {

	public double x;
	public double y;
	public double z;

	public double xlen = .3;
	public double ylen = .3;
	public double zlen = 1.4;

	private double side_dx = 0;
	private double side_dy = 0;

	private double forward_dx = 0;
	private double forward_dy = 0;

	public double dz = 0;

	private double headFractionXlen = .5;
	private double headFractionYlen = .5;
	private double headFractionZlen = .75;

	public double headXRot = 0;
	public double headZRot = 0;

	@Override
	public double getCameraX() {
		return this.x + this.xlen * this.headFractionXlen;
	}

	@Override
	public double getCameraY() {
		return this.y + this.ylen * this.headFractionYlen;
	}

	@Override
	public double getCameraZ() {
		return this.z + this.zlen * this.headFractionZlen;
	}

	@Override
	public double getCameraXRot() {
		return this.headXRot + 90;
	}

	@Override
	public double getCameraYRot() {
		return 0;
	}

	@Override
	public double getCameraZRot() {
		return this.headZRot;
	}

	@Override
	public double getX() {
		return this.x;
	}

	@Override
	public double getY() {
		return this.y;
	}

	@Override
	public double getZ() {
		return this.z;
	}
	
	@Override
	public void getXYZ(Vector3d v) {
		v.x = x;
		v.y = y;
		v.z = z;
	}

	@Override
	public double getWidth() {
		return this.xlen;
	}

	@Override
	public double getHeight() {
		return this.ylen;
	}

	@Override
	public double getDepth() {
		return this.zlen;
	}

	private ThingBridge thingBridge;

	private Inventory inventory = new Inventory();

	public Player(double x, double y, double z, double xRot, double zRot,
			ThingBridge thingBridge) {
		this.x = x;
		this.y = y;
		this.z = z;
		this.headXRot = xRot;
		this.headZRot = zRot;
		this.thingBridge = thingBridge;
	}

	@Override
	public Vector3d getReqMoveVector(double fps) {
		Vector3d v = new Vector3d();
		double dragZ = 1;
		double dragXY = 1;
		if (this.thingBridge.inWater(this)) {
			dragZ = .5;
			dragXY = .75;
		}
		v.x = dragXY * (this.side_dx + this.forward_dx) / fps;
		v.y = dragXY * (this.side_dy + this.forward_dy) / fps;
		v.z = dragZ * this.dz / fps;
		return v;
	}

	@Override
	public void applyMoveVector(Vector3d v) {
		this.x += v.x;
		this.y += v.y;
		this.z += v.z;
	}

	public void lookVertical(double dx) {
		this.headXRot += dx;
		if (this.headXRot < -90)
			this.headXRot = -90;
		if (this.headXRot > 90)
			this.headXRot = 90;
	}

	public void lookHorizontal(double dx) {
		this.headZRot += dx;
	}

	public void setForwardSpeed(double speed) {
		this.side_dx = speed * Math.cos(Math.toRadians(90 + this.headZRot));
		this.side_dy = speed * Math.sin(Math.toRadians(90 + this.headZRot));
	}

	public void setSidewaysSpeed(double speed) {
		this.forward_dx = speed * Math.cos(Math.toRadians(this.headZRot));
		this.forward_dy = speed * Math.sin(Math.toRadians(this.headZRot));
	}

	public void jump(double mag) {
		if (this.thingBridge.onGround(this) || this.thingBridge.inWater(this))
			this.dz = mag;
	}

	@Override
	public void accelerate(double fps, double dx, double dy, double dz) {
		this.dz += dz / fps;
		if (this.thingBridge.inWater(this) && Math.abs(this.dz) > 5)
			this.dz = Math.signum(this.dz) * 5;
	}

	@Override
	public void stopX() {

	}

	@Override
	public void stopY() {

	}

	@Override
	public void stopZ() {
		this.dz = 0;
	}

	@Override
	public void addBlock(Block b) {
		this.inventory.addBlock(b);
	}

	@Override
	public void rmBlock(Block b) {
		this.inventory.useBlock(b.getClass());
	}

	public Vector3d getDirectionLooking() {
		double x = Math.cos(Math.toRadians(90 + this.headZRot))
				* Math.cos(Math.toRadians(this.headXRot));
		double y = Math.sin(Math.toRadians(90 + this.headZRot))
				* Math.cos(Math.toRadians(this.headXRot));
		double z = Math.sin(Math.toRadians(this.headXRot));
		return new Vector3d(x, y, z);
	}

	public boolean isUnderwater() {
		return this.thingBridge.isUnderwater(getCameraX(),
				getCameraY(), getCameraZ());
	}

	@Override
	public void render(GL2 gl) {

		Vector3d dir = getDirectionLooking();
		dir.times(10);

		Vector3i closestBlock = this.thingBridge
				.getClosestNonEmptyBlockOnVector(
						new Vector3d(getCameraX(), getCameraY(), getCameraZ()), dir);

		Vector3i closestEmptyBlock = this.thingBridge
				.getClosestEmptyBlockOnVector(new Vector3d(getCameraX(),
						getCameraY(), getCameraZ()), dir);

		if (closestBlock != null) {
			Face f = HighlightBlockFace.getBetweenFace(closestBlock,
					closestEmptyBlock);
			HighlightBlockFace.highlight(gl, closestBlock, f);
		}
	}

	public boolean createBlock = false;
	public boolean deleteBlock = false;
	public Class blockToPlace = null;

	public Block getBlockToPlace() {
		if (this.blockToPlace == BlockDebug.class){
			return BlockDebug.Make(BlockDebug.MAX_HEALTH);
		}
		else if (this.blockToPlace == BlockDirt.class){
			return BlockDirt.Make(BlockDirt.MAX_HEALTH, false);
		}
		else if (this.blockToPlace == BlockGlass.class){
			return BlockGlass.Make(0, BlockGlass.MAX_HEALTH);
		}
		else if (this.blockToPlace == BlockLeaf.class){
			return BlockLeaf.Make(0, BlockLeaf.MAX_HEALTH);
		}
		else if (this.blockToPlace == BlockSeed.class){
			return BlockSeed.Make(5, BlockSeed.MAX_HEALTH);
		}
		else if (this.blockToPlace == BlockStone.class){
			return BlockStone.Make(BlockStone.MAX_HEALTH);
		}
		else if (this.blockToPlace == BlockWater.class){
			return BlockWater.Make(0, 0, 8);
		}
		else if (this.blockToPlace == BlockWood.class){
			return BlockWood.Make(BlockWoodData.deadBlockWoodData);
		}
		else {
			try {
				throw new Exception("can't create block, unknown block type");
			} catch (Exception e) {
				e.printStackTrace();
				System.exit(1);
			}
			return null;
		}
	}

	public int getBlockAvailability(Class c) {
		return this.inventory.numBlock(c);
	}

	@Override
	public BlockAction getBlockAction() {
		Vector3d dir = getDirectionLooking();
		dir.times(10);

		if (this.createBlock) {
			this.createBlock = false;
			if (!this.inventory.hasBlock(this.blockToPlace))
				return null;
			Vector3i closestBlock = this.thingBridge
					.getClosestEmptyBlockOnVector(
							new Vector3d(getCameraX(), getCameraY(),
									getCameraZ()), dir);
			if (closestBlock != null) {
				AddBlock ab = new AddBlock();
				ab.x = closestBlock.x;
				ab.y = closestBlock.y;
				ab.z = closestBlock.z;
				ab.block = getBlockToPlace();
				return ab;
			} else {
				return null;
			}
		} else if (this.deleteBlock) {
			Vector3i closestBlock = this.thingBridge
					.getClosestNonEmptyBlockOnVector(
							new Vector3d(getCameraX(), getCameraY(),
									getCameraZ()), dir);
			this.deleteBlock = false;
			if (closestBlock != null) {
				RemoveBlock ab = new RemoveBlock();
				ab.x = closestBlock.x;
				ab.y = closestBlock.y;
				ab.z = closestBlock.z;
				return ab;
			} else {
				return null;
			}
		} else {
			return null;
		}
	}
}
