package Hexel.things;

import Hexel.blocks.types.Block;
import Hexel.blocks.types.BlockEmpty;
import Hexel.blocks.types.BlockWater;
import Hexel.chunk.Chunks;
import Hexel.math.HexGeometry;
import Hexel.math.Vector2d;
import Hexel.math.Vector3d;
import Hexel.math.Vector3i;

public class ThingBridge {
	private Chunks chunks;

	private ThingTools thingTools;

	public ThingTools.FixOffsetTmps fixOffsetTmps;

	public ThingBridge(Chunks chunks) {
		this.chunks = chunks;
		this.thingTools = new ThingTools(chunks);
		this.fixOffsetTmps = this.thingTools.new FixOffsetTmps();
	}

	private static final Vector3d onGroundOffset = new Vector3d(0, 0, -.1);
	private static final Vector3d zeroOffset = new Vector3d(0, 0, 0);

	public boolean onGround(Volumetric v) {
		return this.thingTools.collidesSolid(v, onGroundOffset, this.fixOffsetTmps);
	}

	public boolean inWater(Volumetric v) {
		return this.thingTools.collidesWater(v, zeroOffset, this.fixOffsetTmps);
	}

	public boolean isUnderwater(double x, double y, double z) {
		Vector3i tmp3i = new Vector3i();
		Vector2d tmp2d = new Vector2d();
		return this.thingTools.collidesWater(x, y, z, tmp2d, tmp3i);
	}

	public Vector3i getClosestNonEmptyBlockOnVector(Vector3d offset, Vector3d t) {
		double STEP_SIZE = .01;

		Vector3d v = new Vector3d();
		Vector3d step = Vector3d.Sub(t, v);
		step.unit();
		step.times(STEP_SIZE);

		Vector2d tmp2d = new Vector2d();
		Vector3i tmp3i = new Vector3i();

		Vector3i blockPos = new Vector3i();

		while (v.mag() < t.mag()) {
			v.add(step);

			HexGeometry.cartesianToBlock(v.x + offset.x, v.y + offset.y, v.z
					+ offset.z, tmp2d, blockPos);

			Block b = this.chunks
					.getBlock(blockPos.x, blockPos.y, blockPos.z, tmp3i);
			if (!(b instanceof BlockEmpty || b instanceof BlockWater)) {
				return blockPos;
			}
		}
		return null;
	}

	public Vector3i getClosestEmptyBlockOnVector(Vector3d offset, Vector3d t) {
		double STEP_SIZE = .01;

		Vector3d v = new Vector3d();
		Vector3d step = Vector3d.Sub(t, v);
		step.unit();
		step.times(STEP_SIZE);

		Vector3i farthestEmpty = new Vector3i();
		Vector3i blockPos = new Vector3i();

		Vector2d tmp2d = new Vector2d();
		Vector3i tmp3i = new Vector3i();

		while (v.mag() < t.mag()) {
			v.add(step);
			HexGeometry.cartesianToBlock(v.x + offset.x, v.y + offset.y, v.z
					+ offset.z, tmp2d, blockPos);

			Block b = this.chunks
					.getBlock(blockPos.x, blockPos.y, blockPos.z, tmp3i);
			if (!(b instanceof BlockEmpty || b instanceof BlockWater)) {
				return farthestEmpty;
			} else {
				farthestEmpty.x = blockPos.x;
				farthestEmpty.y = blockPos.y;
				farthestEmpty.z = blockPos.z;
			}
		}
		return null;
	}
}
