package Hexel.things;

import java.util.HashSet;
import java.util.Iterator;

import Hexel.Resources;
import Hexel.blocks.BlockTools;
import Hexel.blocks.types.Block;
import Hexel.blocks.types.BlockEmpty;
import Hexel.chunk.Chunks;
import Hexel.math.Vector2i;
import Hexel.math.Vector3d;
import Hexel.math.Vector3i;
import Hexel.things.blockManipulation.AddBlock;
import Hexel.things.blockManipulation.BlockAction;
import Hexel.things.blockManipulation.BlockManipulator;
import Hexel.things.blockManipulation.RemoveBlock;

public class ThingSimulator {

	private HashSet<Thing> things = new HashSet<Thing>();

	private Chunks chunks;

	private ThingBridge thingBridge;

	public ThingBridge getThingBridge() {
		return this.thingBridge;
	}

	private ThingTools thingTools;

	public ThingTools.FixOffsetTmps fixOffsetTmps;

	public ThingSimulator(Chunks chunks) {
		this.chunks = chunks;
		this.thingBridge = new ThingBridge(chunks);
		this.thingTools = new ThingTools(chunks);
		this.fixOffsetTmps = this.thingTools.new FixOffsetTmps();
	}

	public void addThing(Thing thing) {
		this.things.add(thing);
	}

	public void step(double fps) {
		Iterator<Thing> iter = this.things.iterator();
		Vector3i tmp = Resources.vector3iResourcePool.aquire();
		Vector2i tmp2 = Resources.vector2iResourcePool.aquire();
		while (iter.hasNext()) {
			Thing thing = iter.next();

			if (thing instanceof Movable) {
				Movable movable = (Movable) thing;
				movable.accelerate(fps, 0, 0, -9.8);
				Vector3d reqMoveVector = movable.getReqMoveVector(fps);
				moveMovable(movable, reqMoveVector, tmp);
			}
			if (thing instanceof BlockManipulator) {
				BlockAction action = ((BlockManipulator) thing)
						.getBlockAction();
				if (action instanceof AddBlock) {
					AddBlock addBlockAction = (AddBlock) action;
					Block original = this.chunks.getBlock(addBlockAction.x,
							addBlockAction.y, addBlockAction.z, tmp);
					this.chunks.setBlock(addBlockAction.x, addBlockAction.y,
							addBlockAction.z, addBlockAction.block, tmp, tmp2);
					if (thing instanceof Volumetric) {
						if (thing instanceof Cuboid) {
							Vector3d offset = Resources.vector3dResourcePool.aquire();
							offset.x = 0;
							offset.y = 0;
							offset.z = 0;
							if (this.thingTools.fixOffset((Cuboid) thing,
									offset, false, this.fixOffsetTmps) != null)
								this.chunks.setBlock(addBlockAction.x,
										addBlockAction.y, addBlockAction.z,
										original, tmp, tmp2);
							else {
								if (thing instanceof InventoryOwner) {
									InventoryOwner inventoryOwner = (InventoryOwner) thing;
									inventoryOwner.rmBlock(addBlockAction.block);
								}
							}
							Resources.vector3dResourcePool.recycle(offset);
						}

					}
				} else if (action instanceof RemoveBlock) {
					RemoveBlock rmBlockAction = (RemoveBlock) action;
					Block b = this.chunks.getBlock(rmBlockAction.x, rmBlockAction.y,
							rmBlockAction.z, tmp);
					if (b.getHealth() < 0) {
						this.chunks.setBlock(rmBlockAction.x, rmBlockAction.y,
								rmBlockAction.z, BlockEmpty.Make(0), tmp, tmp2);
						if (thing instanceof InventoryOwner) {
							InventoryOwner inventoryOwner = (InventoryOwner) thing;
							inventoryOwner.addBlock(b);
						}
					} else {
						Block newB = BlockTools.updateHealth(b, b.getHealth()-1);
						this.chunks.setBlock(rmBlockAction.x, rmBlockAction.y,
								rmBlockAction.z, newB, tmp, tmp2);
					}

				}
			}

		}
		Resources.vector3iResourcePool.recycle(tmp);
		Resources.vector2iResourcePool.recycle(tmp2);
		// System.out.System.out.println(((b-a) + " " + (c-b)));
	}

	public boolean equal(double a, double b, double epsilon) {
		return Math.abs(a - b) < epsilon;
	}
	
	public void moveMovable(Movable movable, Vector3d reqMoveVector, Vector3i tmp){

		Vector3d partialMove = new Vector3d(reqMoveVector);
		
		partialMove.unit();
		partialMove.times(.1);
		Vector3d leftToMove = new Vector3d(reqMoveVector);
		while (leftToMove.mag() > 0){
			if (leftToMove.mag() > partialMove.mag()){
				Movement movement = getMovement((Volumetric) movable,
						partialMove, tmp);
				if (movement.stoppedZ) {
					movable.stopZ();
				}
				movable.applyMoveVector(movement);
				leftToMove.sub(partialMove);
			}
			else {
				Movement movement = getMovement((Volumetric) movable,
						leftToMove, tmp);
				if (movement.stoppedZ) {
					movable.stopZ();
				}
				movable.applyMoveVector(movement);
				leftToMove.sub(leftToMove);
			}
		}
		
	}

	public Movement getMovement(Volumetric v, Vector3d reqMoveVector,
			Vector3i tmp) {
		Movement m = new Movement(reqMoveVector);
		
		Vector3d toFix = null;
		if (v instanceof Cuboid) {
			toFix = this.thingTools.fixOffset(new PaddedCuboid((Cuboid)v, .1, .1, 0), reqMoveVector, true, this.fixOffsetTmps);
		}

		if (toFix != null) {
			m.add(toFix);
			
			Vector3d afterFix = this.thingTools.fixOffset((Cuboid) v, new Vector3d(m.x, m.y, m.z-.1), true, this.fixOffsetTmps);
			m.stoppedZ = afterFix != null;
		}

		return m;
	}
}
