package Hexel.things;

import java.util.HashSet;

import Hexel.Resources;
import Hexel.blocks.types.Block;
import Hexel.blocks.types.BlockEmpty;
import Hexel.blocks.types.BlockWater;
import Hexel.chunk.Chunks;
import Hexel.math.HexGeometry;
import Hexel.math.Vector2d;
import Hexel.math.Vector2i;
import Hexel.math.Vector3d;
import Hexel.math.Vector3i;

public class ThingTools {

	private Chunks chunks;

	public ThingTools(Chunks chunks) {
		this.chunks = chunks;
	}

	private interface CheckCollidableBlock {
		public boolean isColliding(Block block);
	}

	private CheckCollidableBlock checkCollidesSolid = new CheckCollidableBlock() {
		@Override
		public boolean isColliding(Block block) {
			return !(block instanceof BlockEmpty || block instanceof BlockWater);
		}
	};

	private CheckCollidableBlock checkCollidesWater = new CheckCollidableBlock() {
		@Override
		public boolean isColliding(Block block) {
			return block instanceof BlockWater;
		}
	};

	public class CollidesTmps {
		public Vector3d[] v3d6 = new Vector3d[6];
		public Vector2d vd = new Vector2d();
		public Vector2d ud = new Vector2d();
		public double[] d6 = new double[6];
		public Vector2d v2d = new Vector2d();

		public final Vector3d up = new Vector3d(0, 0, 1);
		public final Vector3d vd100 = new Vector3d(1, 0, 0);
		public final Vector3d vd010 = new Vector3d(0, 1, 0);
		public final Vector3d vd001 = new Vector3d(0, 0, 1);

		public Vector3d[] v3d3 = new Vector3d[]{ new Vector3d(), new Vector3d(), new Vector3d() };

	}

	public class FixOffsetTmps {
		public Vector3i tmp3i = new Vector3i();
		public Vector2d tmp2d = new Vector2d();
		public HashSet<Vector3i> toCheck = new HashSet<Vector3i>();
		public HashSet<Vector3d> toFixBlockSet = new HashSet<Vector3d>();
		public Vector3d attempt = new Vector3d();
		public Vector3i tmp = new Vector3i();
		public CollidesTmps collidesTmps = new CollidesTmps();


	}

	public boolean collidesSolid(Volumetric v, Vector3d offset, FixOffsetTmps tmps) {
		if (v instanceof Cuboid) {
			return this.fixOffset((Cuboid) v, offset, false, this.checkCollidesSolid, tmps) != null;
		} else {
			return false;
		}
	}

	public boolean collidesWater(Volumetric v, Vector3d offset, FixOffsetTmps tmps) {
		if (v instanceof Cuboid) {
			return this.fixOffset((Cuboid) v, offset, false, this.checkCollidesWater, tmps) != null;
		} else {
			return false;
		}
	}

	public boolean collidesWater(double x, double y, double z, Vector2d tmp2d, Vector3i tmp3i) {
		Vector3i blockPos = new Vector3i();
		HexGeometry.cartesianToBlock(x, y, z, tmp2d, blockPos);
		Block b = this.chunks.getBlock(blockPos.x, blockPos.y, blockPos.z, tmp3i);
		return b instanceof BlockWater;
	}

	public Vector3d fixOffset(Cuboid c, Vector3d offset, boolean considerNeighbors, FixOffsetTmps tmps) {
		return fixOffset(c, offset, considerNeighbors, this.checkCollidesSolid, tmps);
	}

	public Vector3d fixOffset(Cuboid c, Vector3d offset,
			boolean considerNeighbors, CheckCollidableBlock checkCollidableBlock, FixOffsetTmps tmps) {
		double width = c.getWidth();
		double height = c.getHeight();
		double depth = c.getDepth();


		tmps.toCheck.clear();

		Vector3i checkBlockPos = tmps.tmp3i;
		double dx = -.1;
		while (true) {
			double dy = -.1;
			while (true) {
				double dz = -.1;
				while (true) {
					double x = c.getX() + dx + offset.x;
					double y = c.getY() + dy + offset.y;
					double z = c.getZ() + dz + offset.z;
					HexGeometry.cartesianToBlock(x, y, z, tmps.tmp2d, checkBlockPos);
					if (!tmps.toCheck.contains(checkBlockPos)){
						Vector3i p = Resources.vector3iResourcePool.aquire();
						p.x = checkBlockPos.x;
						p.y = checkBlockPos.y;
						p.z = checkBlockPos.z;
						tmps.toCheck.add(p);
					}

					if (dz == depth + .1)
						break;
					dz = Math.min(depth + .1, dz + .05);
				}
				if (dy == height + .1)
					break;
				dy = Math.min(height + .1, dy + .05);
			}
			if (dx == width + .1)
				break;
			dx = Math.min(width + .1, dx + .05);

		}

		tmps.toFixBlockSet.clear();
		for (Vector3i blockPos : tmps.toCheck) {
			Block b = this.chunks.getBlock(blockPos.x, blockPos.y, blockPos.z, tmps.tmp3i);
			if (checkCollidableBlock.isColliding(b)) {
				Vector3d partialFix = Resources.vector3dResourcePool.aquire();
				partialFix.x = 0;
				partialFix.y = 0;
				partialFix.z = 0;
				collides(c, offset, blockPos.x, blockPos.y, blockPos.z,
						partialFix, tmps.collidesTmps);
				tmps.toFixBlockSet.add(partialFix);
			}
			Resources.vector3iResourcePool.recycle(blockPos);
		}


		Vector3d toFix = new Vector3d();
		for (Vector3d toFixBlock : tmps.toFixBlockSet) {
			toFix.add(toFixBlock);
			Resources.vector3dResourcePool.recycle(toFixBlock);
		}
		if (toFix.mag() == 0)
			return null;
		else {
				return toFix;
		}
	}

	public void collides(Cuboid c, Vector3d offset, int blockx, int blocky,
			int blockz, Vector3d toFix, CollidesTmps tmps) {

		Vector2i[] points;
		if (blockx % 2 == 0) {
			points = HexGeometry.evenTringlePoints;
		} else {
			points = HexGeometry.oddTringlePoints;
		}

		Vector3d[] axis = tmps.v3d6;
		axis[0] = tmps.vd100;
		axis[1] = tmps.vd010;
		axis[2] = tmps.vd001;
		axis[3] = null;
		axis[4] = null;
		axis[5] = null;


		Vector3d up = tmps.up;
		for (int i = 0; i < points.length; i++) {
			Vector2i v = points[i];
			Vector2i u = points[(i + 1) % points.length];
			Vector2d vd = tmps.vd;
			Vector2d ud = tmps.ud;
			HexGeometry.hexToCartesian(v.x, v.y, vd);
			HexGeometry.hexToCartesian(u.x, u.y, ud);

			Vector3d plane = tmps.v3d3[i];
			plane.x = vd.x - ud.x;
			plane.y = vd.y - ud.y;
			plane.z = 0;
			plane.cross(up);
			plane.unit();
			axis[3 + i] = plane;
			if (plane.y == -1)
				plane.y = 1;
		}
		double[] magnitudes = tmps.d6;
		magnitudes[0] = 0;
		magnitudes[1] = 0;
		magnitudes[2] = 0;
		magnitudes[3] = 0;
		magnitudes[4] = 0;
		magnitudes[5] = 0;
		for (int i = 0; i < axis.length; i++) {
			Double m = calcOverlapMagnitude(c, offset, blockx, blocky, blockz,
					axis[i], tmps);
			if (m == 0)
				return;
			magnitudes[i] = m;

		}
		for (int i = 0; i < axis.length; i++) {
			double overlapMagnitude = magnitudes[i];
			if (overlapMagnitude != 0
					&& (toFix.mag() == 0 || Math.abs(overlapMagnitude) < toFix.mag())) {
				toFix.x = overlapMagnitude * axis[i].x;
				toFix.y = overlapMagnitude * axis[i].y;
				toFix.z = overlapMagnitude * axis[i].z;
			}
		}
	}

	public Double calcOverlapMagnitude(Cuboid c, Vector3d offset, int blockx,
			int blocky, int blockz, Vector3d axis, CollidesTmps tmps) {
		double minC = Integer.MAX_VALUE;
		double maxC = Integer.MIN_VALUE;
		double cx = c.getX() + offset.x;
		double cy = c.getY() + offset.y;
		double cz = c.getZ() + offset.z;
		minC = Math.min(minC, axis.dot(cx, cy, cz));
		minC = Math.min(minC, axis.dot(cx + c.getWidth(), cy, cz));
		minC = Math.min(minC, axis.dot(cx, cy + c.getHeight(), cz));
		minC = Math.min(minC,
				axis.dot(cx + c.getWidth(), cy + c.getHeight(), cz));
		minC = Math.min(minC, axis.dot(cx, cy, cz + c.getDepth()));
		minC = Math.min(minC,
				axis.dot(cx + c.getWidth(), cy, cz + c.getDepth()));
		minC = Math.min(minC,
				axis.dot(cx, cy + c.getHeight(), cz + c.getDepth()));
		minC = Math.min(
				minC,
				axis.dot(cx + c.getWidth(), cy + c.getHeight(),
						cz + c.getDepth()));

		maxC = Math.max(maxC, axis.dot(cx, cy, cz));
		maxC = Math.max(maxC, axis.dot(cx + c.getWidth(), cy, cz));
		maxC = Math.max(maxC, axis.dot(cx, cy + c.getHeight(), cz));
		maxC = Math.max(maxC,
				axis.dot(cx + c.getWidth(), cy + c.getHeight(), cz));
		maxC = Math.max(maxC, axis.dot(cx, cy, cz + c.getDepth()));
		maxC = Math.max(maxC,
				axis.dot(cx + c.getWidth(), cy, cz + c.getDepth()));
		maxC = Math.max(maxC,
				axis.dot(cx, cy + c.getHeight(), cz + c.getDepth()));
		maxC = Math.max(
				maxC,
				axis.dot(cx + c.getWidth(), cy + c.getHeight(),
						cz + c.getDepth()));

		double minB = Integer.MAX_VALUE;
		double maxB = Integer.MIN_VALUE;

		Vector2d p = tmps.v2d;

		Vector2i[] points;
		if (blockx % 2 == 0) {
			points = HexGeometry.evenTringlePoints;
		} else {
			points = HexGeometry.oddTringlePoints;
		}

		for (Vector2i point : points) {
			HexGeometry.hexToCartesian(Math.floor(blockx / 2.0) + point.x,
					blocky + point.y, p);

			minB = Math.min(minB, axis.dot(p.x, p.y, blockz / 2.0));
			maxB = Math.max(maxB, axis.dot(p.x, p.y, blockz / 2.0));

			minB = Math.min(minB, axis.dot(p.x, p.y, blockz / 2.0 + .5));
			maxB = Math.max(maxB, axis.dot(p.x, p.y, blockz / 2.0 + .5));
		}

		if (maxC <= minB || minC >= maxB) {
			return 0.0;
		} else {
			double a = minB - maxC;
			double b = maxB - minC;

			if (Math.abs(a) > Math.abs(b))
				return b;
			else
				return a;
		}
	}
}
